DebuffCounter = LibStub("AceAddon-3.0"):NewAddon("DebuffCounter", "AceConsole-3.0", "AceEvent-3.0", "AceTimer-3.0")
local L = LibStub("AceLocale-3.0"):GetLocale("DebuffCounter")

local DC_VERSION = GetAddOnMetadata("DebuffCounter", "Version") 

local DC_SNDFILE = "Sound\\Doodad\\G_GongTroll01.wav"
local DC_LIMIT = 40
local DC_WCD = 5
local DC_TCD = true
local DC_FCD = true
local DC_SCD = true

local options = {
    name = "DebuffCounter",
    handler = DebuffCounter,
    type = "group",
    args = {
		enable={
	      name=L["Enable"],
	      desc=L["Enables / disables the addon"],
		  order=1,
	      type="toggle",
		  get="GetEnabled",
		  set="SetEnabled"
		},
		reset={
		  name=L["Reset to defaults"],
		  desc=L["Reset addon to default settings"],
		  order=10,
		  type="execute",
		  func="Reset"
		},
		version={
		  name=L["Version"] .. " " .. DC_VERSION .. " -- " .. L["Terrok @ Sentinels(US) Horde"],
		  order=-1,
		  type="header"
		 }
	}
}

options.args.frameopts = {
	name=L["Frame Options"],
	order=2,
	type="group",
	inline=true,
	args={
		showtarget={
	      name=L["Show Target"],
	      desc=L["Shows # of Debuffs on Target"],
		  order=1,
	      type="toggle",
		  get="GetShowTarget",
		  set="SetShowTarget"
		},
		showfocus={
	      name=L["Show Focus"],
	      desc=L["Shows # of Debuffs on Focus"],
		  order=2,
	      type="toggle",
		  get="GetShowFocus",
		  set="SetShowFocus"
		},
		showit={
		  name=L["Show the frame(s)"],
		  desc=L["Choose whether to show the window counter only in Party/Raid"],
		  order=3,
		  type="select",
		  values={
		    a_always=L["Always"],
		    b_party=L["In Party/Raid"],
		    c_raid=L["In Raid only"]},
		  get="GetShowIt",
		  set="SetShowIt"
		},
		scale={
		  name=L["Frame Scale"],
		  desc=L["Set the scale of the frame(s)"],
		  order=4,
		  type="range",
		  min=0.5,
		  max=2,
		  step=0.01,
		  bigStep=0.05,
		  isPercent=true,
		  get="GetScale",
		  set="SetScale"
		},
		alpha={
		  name=L["Frame Transparency"],
		  desc=L["Set the transparency of the frame(s)"],
		  order=5,
		  type="range",
		  min=0.25,
		  max=1,
		  step=0.01,
		  bigStep=0.01,
		  isPercent=true,
		  get="GetAlpha",
		  set="SetAlpha"
		},
		resetframe={
		  name=L["Reset frame positions"],
		  desc=L["Reset the frame(s) to their default positions"],
		  order=6,
		  type="execute",
		  func="ResetFramePos"
		}
	}
}

options.args.warnopts = {
	name=L["Warning Options"],
	order=3,
	type="group",
	inline=true,
	args={
		showwarn={
			name=L["Show warning:"],
			desc=L["Choose whether to show a warning when the Debuff limit is reached."],
			order=1,
			type="select",
			values={
				a_none=L["None"],
				b_rw=L["In Raid Warning"],
				c_chat=L["In Chat"]
			},
			get="GetShowWarn",
			set="SetShowWarn"
		},
		showwarnfor={
			name=L["Show warning for:"],
			desc=L["Choose whether to show the warning for your Target, Focus, or both."],
			order=2,
			type="select",
			values={
				a_targetonly=L["Target only"],
				b_focusonly=L["Focus only"],
				c_both=L["Both"]
			},
			get="GetShowWarnFor",
			set="SetShowWarnFor"
		},
		playsound={
			name=L["Play sound"],
			desc=L["Choose whether to play an audible warning when the Debuff limit is reached."],
			order=3,
			type="toggle",
			get="GetPlaySound",
			set="SetPlaySound"
		}
	}
}

local defaults = {
    profile = {
        enable = true,
		showtarget = true,
		showfocus = false,
		showit = "a_always",
		Tpoint = "BOTTOMRIGHT",
		TrelativePoint = "CENTER",
		Tx = 0,
		Ty = 20,
		Fpoint = "BOTTOMLEFT",
		FrelativePoint = "CENTER",
		Fx = 0,
		Fy = 20,
		scale = 1,
		alpha = 1,
		showwarn = "b_rw",
		showwarnfor = "a_targetonly",
		playsound = true
    }
}

function DebuffCounter:OnInitialize()
    -- Called when the addon is loaded
    self.db = LibStub("AceDB-3.0"):New("DebuffCounter", defaults)
    LibStub("AceConfig-3.0"):RegisterOptionsTable("DebuffCounter", options)
    self.optionsFrame = LibStub("AceConfigDialog-3.0"):AddToBlizOptions("DebuffCounter", L["DebuffCounter"])
    self:RegisterChatCommand(L["dc"], "ChatCommand")
    self:RegisterChatCommand(L["debuffcounter"], "ChatCommand")
	self:CreateTargetFrame()
	self:CreateFocusFrame()
	self:SetFramePos()
	self.TargetDebuffs = 0
	self.FocusDebuffs = 0
	self:Print(L["DebuffCounter"] .. " " .. DC_VERSION .. " " .. L["Initialized. Type /dc for options"])
end

function DebuffCounter:OnEnable()
    -- Called when the addon is enabled
    self:RegisterEvent("UNIT_AURA")
	self:RegisterEvent("PLAYER_TARGET_CHANGED")
	self:RegisterEvent("PLAYER_FOCUS_CHANGED")
	self:RegisterEvent("PARTY_MEMBERS_CHANGED")
	self:RegisterEvent("RAID_ROSTER_UPDATE")
	self:UpdateFrames()
	self:Print(L["Enabled"])
end

function DebuffCounter:OnDisable()
    -- Called when the addon is enabled
	self:UpdateFrames()
	self:Print(L["Disabled"])
end

--Timer Functions

function DebuffCounter:SetTCD()
	DC_TCD = false
	self:ScheduleTimer("ResetTCD", DC_WCD)
end

function DebuffCounter:ResetTCD()
	DC_TCD = true
end

function DebuffCounter:SetFCD()
	DC_FCD = false
	self:ScheduleTimer("ResetFCD", DC_WCD)
end

function DebuffCounter:ResetFCD()
	DC_FCD = true
end

function DebuffCounter:SetSCD()
	DC_SCD = false
	self:ScheduleTimer("ResetSCD", DC_WCD)
end

function DebuffCounter:ResetSCD()
	DC_SCD = true
end

--Event Functions

function DebuffCounter:UNIT_AURA(info, unit)
	if (unit == "target") then
		--Target status changed, count debuffs
		self.TargetDebuffs = self:CountDebuffs(unit)
	elseif (unit == "focus") then
		--Focus status changed, count debuffs
		self.FocusDebuffs = self:CountDebuffs(unit)
	end
	self:UpdateFrames()
end

function DebuffCounter:PLAYER_TARGET_CHANGED()
	self.TargetDebuffs = self:CountDebuffs("target")
	self:UpdateFrames()
end

function DebuffCounter:PLAYER_FOCUS_CHANGED()
	self.FocusDebuffs = self:CountDebuffs("focus")
	self:UpdateFrames()
end

function DebuffCounter:PARTY_MEMBERS_CHANGED()
	self:UpdateVisible()
end

function DebuffCounter:RAID_ROSTER_UPDATE()
	self:UpdateVisible()
end

--Custom Functions

function DebuffCounter:CountDebuffs(unit)
	local i = 1
	while true do
		local name, rank, texture, count, type, duration, timeleft = UnitDebuff(unit, i);
		if not name then
			return i - 1; -- There are no more buffs / debuffs, exit infinite loop
		end
		i = i + 1; -- Go for the next index		
	end
end

function DebuffCounter:UpdateFrames()
	self.TFrameT:SetText(tostring(self.TargetDebuffs))
	self.TFrameT:SetTextColor(1,(1-((1/DC_LIMIT)*self.TargetDebuffs)),(1-((1/DC_LIMIT)*self.TargetDebuffs)))
	self.FFrameT:SetText(tostring(self.FocusDebuffs))
	self.FFrameT:SetTextColor(1,(1-((1/DC_LIMIT)*self.FocusDebuffs)),(1-((1/DC_LIMIT)*self.FocusDebuffs)))
	self:UpdateWarning()
	self:UpdateAlphaScale()
	self:UpdateVisible()
end

function DebuffCounter:UpdateVisible()
	if (not self.db.profile.enable) then
		self.TFrame:Hide()
		self.FFrame:Hide()
		return false
	end
	
	local starget = true
	local sfocus = true
	
	if (not self.db.profile.showtarget) then
		starget = false
	end
	
	if(not self.db.profile.showfocus) then
		sfocus = false
	end
	
	if(GetNumRaidMembers() == 0) then
	--Not in Raid
		if (GetNumPartyMembers() == 0) then
		--Not in Party
			if (self.db.profile.showit ~= "a_always") then
				starget = false
				sfocus = false
			end
		else
		--In Party
			if (self.db.profile.showit == "c_raid") then
				starget = false
				sfocus = false
			end
		end
	end
	
	if (starget) then
		self.TFrame:Show()
	else
		self.TFrame:Hide()
	end
	
	if (sfocus) then
		self.FFrame:Show()
	else
		self.FFrame:Hide()
	end
end

function DebuffCounter:UpdateAlphaScale()
	self.TFrame:SetScale(self.db.profile.scale)
	self.FFrame:SetScale(self.db.profile.scale)
	self.TFrame:SetAlpha(self.db.profile.alpha)
	self.FFrame:SetAlpha(self.db.profile.alpha)
end

function DebuffCounter:UpdateWarning()
	if (not self.db.profile.enable) then
		return false
	end
	
	local tmessage = false
	local fmessage = false
	local playsnd = false
	
	if ((self.TargetDebuffs >= DC_LIMIT) and (self.db.showwarnfor ~= "b_focusonly")) then
		tmessage = L["Target \""] .. GetUnitName("target") .. L["\" has reached the Debuff limit!"]
		playsnd = true
	end
	
	if ((self.FocusDebuffs >= DC_LIMIT) and (self.db.showwarnfor ~= "a_targetonly")) then
		fmessage = L["Focus \""] .. GetUnitName("focus") .. L["\" has reached the Debuff limit!"]
		playsnd = true
	end
	
	if (self.db.profile.showwarn == "b_rw") then
		if (tmessage and DC_TCD) then
			self:SetTCD()
			RaidNotice_AddMessage(RaidWarningFrame, tmessage, {r=1,g=0,b=0})
		end
		if (fmessage and DC_FCD) then
			self:SetFCD()
			RaidNotice_AddMessage(RaidWarningFrame, fmessage, {r=1,g=0,b=0})
		end
	elseif (self.db.profile.showwarn == "c_chat") then
		if (tmessage and DC_TCD) then
			self:SetTCD()
			self:Print("|cffff0000" .. tmessage)
		end
		if (fmessage and DC_FCD) then
			self:SetFCD()
			self:Print("|cffff0000" .. fmessage)
		end
	end
	
	if (self.db.profile.playsound and playsnd and DC_SCD) then
		self:SetSCD()
		PlaySoundFile(DC_SNDFILE)
	end
end

function DebuffCounter:SaveFramePos()
	local point,relativeTo,relativePoint,x,y = self.TFrame:GetPoint()
	self.db.profile.Tpoint = point
	self.db.profile.TrelativePoint = relativePoint
	self.db.profile.Tx = x
	self.db.profile.Ty = y
	
	local point,relativeTo,relativePoint,x,y = self.FFrame:GetPoint()
	self.db.profile.Fpoint = point
	self.db.profile.FrelativePoint = relativePoint
	self.db.profile.Fx = x
	self.db.profile.Fy = y
end

function DebuffCounter:SetFramePos()
--Set framepos to profile position
	self.TFrame:ClearAllPoints()
	self.TFrame:SetPoint( self.db.profile.Tpoint, UIParent, self.db.profile.TrelativePoint, self.db.profile.Tx, self.db.profile.Ty)
	
	self.FFrame:ClearAllPoints()
	self.FFrame:SetPoint( self.db.profile.Fpoint, UIParent, self.db.profile.FrelativePoint, self.db.profile.Fx, self.db.profile.Fy)
end

function DebuffCounter:ResetFramePos()
--Set framepos to default position
	self.TFrame:ClearAllPoints()
	self.TFrame:SetPoint( defaults.profile.Tpoint, UIParent, defaults.profile.TrelativePoint, defaults.profile.Tx, defaults.profile.Ty)
	
	self.FFrame:ClearAllPoints()
	self.FFrame:SetPoint( defaults.profile.Fpoint, UIParent, defaults.profile.FrelativePoint, defaults.profile.Fx, defaults.profile.Fy)
end

--Options Functions

function DebuffCounter:ChatCommand(input)
    InterfaceOptionsFrame_OpenToFrame(self.optionsFrame)
end

function DebuffCounter:GetEnabled(info)
	return self.db.profile.enable
end

function DebuffCounter:SetEnabled(info,value)
	self.db.profile.enable = value
	if value then
		self:Enable()
		options.args.frameopts.disabled = false
		options.args.warnopts.disabled = false
	else
		self:Disable()
		options.args.frameopts.disabled = true
		options.args.warnopts.disabled = true
	end
end
function DebuffCounter:GetShowTarget(info)
	return self.db.profile.showtarget
end

function DebuffCounter:SetShowTarget(info, value)
	if value then
		self.db.profile.showtarget = true
		self:UpdateFrames()
	else
		self.db.profile.showtarget = false
		self:UpdateFrames()
	end
end

function DebuffCounter:GetShowFocus(info)
	return self.db.profile.showfocus
end

function DebuffCounter:SetShowFocus(info, value)
	if value then
		self.db.profile.showfocus = true
		self:UpdateFrames()
	else
		self.db.profile.showfocus = false
		self:UpdateFrames()
	end
end

function DebuffCounter:GetShowIt(info)
	return self.db.profile.showit
end

function DebuffCounter:SetShowIt(info, value)
	self.db.profile.showit=value
	self:UpdateFrames()
end

function DebuffCounter:GetScale(info)
	return self.db.profile.scale
end

function DebuffCounter:SetScale(info, value)
	self.db.profile.scale=value
	self:UpdateAlphaScale()
end

function DebuffCounter:GetAlpha(info)
	return self.db.profile.alpha
end

function DebuffCounter:SetAlpha(info, value)
	self.db.profile.alpha=value
	self:UpdateAlphaScale()
end

function DebuffCounter:GetShowWarn(info)
	return self.db.profile.showwarn
end

function DebuffCounter:SetShowWarn(info, value)
	self.db.profile.showwarn=value
	if (self.db.profile.showwarn == "a_none") then
		options.args.warnopts.args.showwarnfor.disabled = true
	else
		options.args.warnopts.args.showwarnfor.disabled = false
	end
end

function DebuffCounter:GetShowWarnFor(info)
	return self.db.profile.showwarnfor
end

function DebuffCounter:SetShowWarnFor(info, value)
	self.db.profile.showwarnfor=value
end

function DebuffCounter:GetPlaySound(info)
	return self.db.profile.playsound
end

function DebuffCounter:SetPlaySound(info,value)
	self.db.profile.playsound = value
end

function DebuffCounter:Reset()
	self.db:ResetProfile()
	self:SetEnabled(nil,true)
	self:SetFramePos()
	self:UpdateFrames()
end





